package test.listeners.github1602;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

public class TestClassWithPassingConfigsSample {
  @BeforeMethod
  public void beforeMethod() {}

  @Test
  public void testMethod() {}

  @AfterMethod
  public void afterMethod() {}
}
