package test.listeners.github2385;

import org.testng.annotations.Test;

public class TestMultiInheritSample implements ITestClassInterface, ITestInterface {

  @Test
  public void testMultiInherit() {}
}
