package test.methodselectors;

import org.testng.annotations.Test;

public class SampleTest {

  @Test(groups = {"test1"})
  public void test1() {}

  @Test(groups = {"test2"})
  public void test2() {}

  @Test(groups = {"test3"})
  public void test3() {}
}
