package org.testng.internal.misamples;

public class Batman implements SuperHeroCapabilities {

  @Override
  public void fly() {}

  @Override
  public void liftWeights() {}

  @Override
  public void yellSlogan() {}
}
