package test.mixed;

import junit.framework.TestCase;

/** @author lukas */
public class JUnit3Test1 extends TestCase {

  public JUnit3Test1(String name) {
    super(name);
  }

  public void testA() {}

  public void testB() {}
}
