package test.serviceloader;

import org.testng.IConfigurationListener;
import org.testng.ITestResult;

public class MyConfigurationListener implements IConfigurationListener {

  @Override
  public void onConfigurationSuccess(ITestResult itr) {}

  @Override
  public void onConfigurationFailure(ITestResult itr) {}

  @Override
  public void onConfigurationSkip(ITestResult itr) {}
}
