package test.testng1396;

import org.testng.annotations.Test;

public class SampleFor1396 {
  public class Sample0For1396 {
    @Test
    public void sample() {}
  }

  @TestNG1396HighPriority
  public class Sample1For1396 {
    @Test
    public void sample() {}
  }

  public class Sample2For1396 {
    @Test
    public void sample() {}
  }

  @TestNG1396HighPriority
  public class Sample3For1396 {
    @Test
    public void sample() {}
  }

  public class Sample4For1396 {
    @Test
    public void sample() {}
  }

  @TestNG1396HighPriority
  public class Sample5For1396 {
    @Test
    public void sample() {}
  }

  public class Sample6For1396 {
    @Test
    public void sample() {}
  }

  @TestNG1396HighPriority
  public class Sample7For1396 {
    @Test
    public void sample() {}
  }

  public class Sample8For1396 {
    @Test
    public void sample() {}
  }

  @TestNG1396HighPriority
  public class Sample9For1396 {
    @Test
    public void sample() {}
  }
}
