package test.listeners.github2385;

import org.testng.annotations.Test;

public class TestMultiLevelInheritSameAnnotationSample implements ITestInterfaceSameInherit {

  @Test
  public void testMultiLevel() {}
}
